package sun.security.krb5.internal;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.HashMap;
import java.util.Date;
import sun.security.krb5.Credentials;
import sun.security.krb5.PrincipalName;
import java.util.Map;

final class ReferralsCache
{
    private static Map<ReferralsCache.ReferralCacheKey, Map<String, ReferralsCache.ReferralCacheEntry>> referralsMap;
    
    ReferralsCache() {
    }
    
    static synchronized void put(PrincipalName principalName, PrincipalName principalName2, String s, String s2, Credentials credentials) {
        ReferralsCache.ReferralCacheKey referralCacheKey = new ReferralCacheKey(principalName, principalName2);
        pruneExpired(referralCacheKey);
        if (credentials.getEndTime().before(new Date())) {
            return;
        }
        Map<String, Object> map = (Map)ReferralsCache.referralsMap.get(referralCacheKey);
        if (map == null) {
            map = new HashMap();
            ReferralsCache.referralsMap.put(referralCacheKey, map);
        }
        map.remove(s);
        ReferralsCache.ReferralCacheEntry o = new ReferralsCache.ReferralCacheEntry(credentials, s2);
        map.put(s, o);
        LinkedList o2 = new LinkedList();
        while (o != null) {
            if (o2.contains(o)) {
                map.remove(o.getToRealm());
                break;
            }
            o2.add(o);
            o = (Object)map.get(o.getToRealm());
        }
        return;
    }
    
    static synchronized ReferralsCache.ReferralCacheEntry get(PrincipalName principalName, PrincipalName principalName2, String s) {
        ReferralsCache.ReferralCacheKey referralCacheKey = new ReferralCacheKey(principalName, principalName2);
        pruneExpired(referralCacheKey);
        Map<Object, Object> map = (Map)ReferralsCache.referralsMap.get(referralCacheKey);
        if (map != null) {
            ReferralsCache.ReferralCacheEntry referralCacheEntry = (ReferralsCache.ReferralCacheEntry)map.get(s);
            if (referralCacheEntry != null) {
                return referralCacheEntry;
            }
        }
        return null;
    }
    
    private static void pruneExpired(ReferralsCache.ReferralCacheKey referralCacheKey) {
        Date when = new Date();
        Map<Object, Object> map = (Map)ReferralsCache.referralsMap.get(referralCacheKey);
        if (map != null) {
            for (Map.Entry entry : map.entrySet()) {
                if (((ReferralsCache.ReferralCacheEntry)entry.getValue()).getCreds().getEndTime().before(when)) {
                    map.remove(entry.getKey());
                }
                else {
                    continue;
                }
            }
        }
    }
    
    static {
        ReferralsCache.referralsMap = new HashMap();
    }
    
    private static final class ReferralCacheKey
    {
        private PrincipalName cname;
        private PrincipalName sname;
        
        ReferralCacheKey(PrincipalName cname, PrincipalName sname) {
            this.cname = cname;
            this.sname = sname;
        }
        
        @Override
        public boolean equals(Object o) {
            if (!(o instanceof ReferralsCache.ReferralCacheKey)) {
                return false;
            }
            ReferralsCache.ReferralCacheKey referralCacheKey = (ReferralsCache.ReferralCacheKey)o;
            return this.cname.equals(referralCacheKey.cname) && this.sname.equals(referralCacheKey.sname);
        }
        
        @Override
        public int hashCode() {
            return this.cname.hashCode() + this.sname.hashCode();
        }
    }
    
    static final class ReferralCacheEntry
    {
        private final Credentials creds;
        private final String toRealm;
        
        ReferralCacheEntry(Credentials creds, String toRealm) {
            this.creds = creds;
            this.toRealm = toRealm;
        }
        
        Credentials getCreds() {
            return this.creds;
        }
        
        String getToRealm() {
            return this.toRealm;
        }
    }
}
